# This module lists down all the functions
# to validate the input files.

import json
import numpy as np
import os
import pandas as pd
import pytest

from olmonk import ConfigDataValidator as CDV

from datum import helpers as hlp
from datum import constants as const
from datum import error_constants as err_const
from datum.exception_wrapper import exception_wrapped, BaseRunner

logs = {'errors': [], 'warnings': {'action': [], 'message': []}}


def validate_via_olmonk(config_dict, input_file_path = None, input_df = None):
    """
    This function checks the file as per the olmonk
    dictionary. Either you provide dataframe or you provide file path.

    Parameters
    ----------
        input_file_path: string
            path of the file.
        input_df: dataframe
            dataframe of the file
        config_dict: dictionary
            olmonk configuration dictionary to validate file
    """
    try:
        file_dict = dict(config_dict)
        if input_file_path:
            file_dict['file_path'] = input_file_path
        else:
            file_dict['df'] = input_df
        cdv = CDV(file_dict)
        cdv.validate()
        return cdv.dv.corrected_df, cdv.dv.logs
    except Exception as e:
        logs = {const.ERRORS: [e.message], const.WARNINGS: {const.ACTION: [],
                                                    const.MESSAGE: []
                                                    }
                }
        return pd.DataFrame(), logs


class BasicValidations(BaseRunner):
    """
    This is a class of basic validation function
    """

    def __init__(self, raw_file, metadata = None, sample_metadata = None,
                config_dict = None):
        """
        """
        self.raw_file_path = raw_file
        self.raw_df = hlp.read_file(self.raw_file_path)
        self.config_dict = config_dict


    def check_file_empty(self):
        """
        This function checks if the file is empty
        """
        try:
            if os.stat(self.raw_file_path).st_size == 0:
                logs[const.ERROR].append("The file is empty")

        except:
            logs[const.ERRORS].append("There is an issue with the file input.")


    def check_data_in_file(self):
        """
        This function checks if the dataframe has the data
        in the file and not just headers
        """
        df = self.raw_df
        col_list = list(df.columns)
        counter = 0
        for col in col_list:
            if(len(df[col]) <= 0):
                counter += 1
            if counter == len(col_list):
                logs[const.ERRORS].append("The input file doesn't contain any value.")


    def check_if_convert_to_df(self):
        """
        This method checks if the file can convert to pandas df.

        :return: Boolean
        """
        try:
            df = self.raw_df
            if df.empty:
                logs[const.ERRORS].append('The input file contains no data')
            else:
                pass
        except Exception as e:
            # :TODO: Implement better catch statement.
            logs[const.ERRORS].append(err_const.FILE_FORMAT_SUPPORT)

    def check_duplicate_rows(self):
        """
        This function deletes all the duplicate all the rows
        from the dataframe
        """
        col_list = list(self.raw_df.columns)
        self.raw_df = self.raw_df.drop_duplicates()
        logs[const.WARNINGS][const.MESSAGE].append(err_const.REMOVED_DUPLICATE_ROWS)


    def check_column_data_types(self):
        """
        This function checks the data types of all the
        mandatory columns from the config_dict and then
        converts them into that data type no matter what the datatype is currently.

        dict : {}
        """
        df = self.raw_df
        data_type_dict = self.config_dict.get('data_types')
        for col, dat_type in list(data_type_dict.items()):
            df[col].apply(str)


    def check_missing_data(self):
        """
        Checks for missing values in the columns provided in
        the config_dict and removes those rows based on the action
        specified.
        """
        df = self.raw_df
        config_dict = self.config_dict
        cols = config_dict['missing']
        for col in cols:
            df = df[~df[col].isnull()]
        hlp.write_file(df, self.raw_file_path)

    def create_log_msg(self, index_list, col, error):
        """
        This function creates a log message in the format of
        olmonk error message i.e.
        -------------
        "Row Number 2 : column Label has invalid label data value , \
        column Label has a required format "C13-Label-1" value"
        ------------

        index_list : list
            list of indexes

        col: string
            column name

        returns
        -------
            error_msg : string
                error string
        """
        error_msgs = []
        for index in index_list:
            error_msgs.append("Row Number {} : column {} has {} value, column {} can't have {} value".\
            format(index, col, error, col, error))
        return error_msgs
